@@ -1,7 +1,9 @@
 //! Module defining the AST (abstract syntax tree).
 
 use super::{ASTFlags, Expr, FnAccess, Stmt};
-use crate::{expose_under_internals, Dynamic, FnNamespace, ImmutableString, Position, ThinVec};
+use crate::{
+    expose_under_internals, Dynamic, FnNamespace, ImmutableString, Position, StaticVec, ThinVec,
+};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 use std::{
@@ -36,7 +38,6 @@ pub struct AST {
 
 impl Default for AST {
     #[inline(always)]
-    #[must_use]
     fn default() -> Self {
         Self::empty()
     }
@@ -749,7 +750,7 @@ impl AST {
                     || !options.intersects(ASTFlags::CONSTANT) && include_variables =>
             {
                 let (name, expr, ..) = &**x;
-                expr.get_literal_value()
+                expr.get_literal_value(None)
                     .map(|value| (name.as_str(), options.intersects(ASTFlags::CONSTANT), value))
             }
             _ => None,
@@ -802,7 +803,6 @@ impl<A: Into<Self>> AddAssign<A> for AST {
 
 impl Borrow<[Stmt]> for AST {
     #[inline(always)]
-    #[must_use]
     fn borrow(&self) -> &[Stmt] {
         self.statements()
     }
@@ -810,7 +810,6 @@ impl Borrow<[Stmt]> for AST {
 
 impl AsRef<[Stmt]> for AST {
     #[inline(always)]
-    #[must_use]
     fn as_ref(&self) -> &[Stmt] {
         self.statements()
     }
@@ -819,7 +818,6 @@ impl AsRef<[Stmt]> for AST {
 #[cfg(not(feature = "no_function"))]
 impl Borrow<crate::Module> for AST {
     #[inline(always)]
-    #[must_use]
     fn borrow(&self) -> &crate::Module {
         self.shared_lib()
     }
@@ -828,7 +826,6 @@ impl Borrow<crate::Module> for AST {
 #[cfg(not(feature = "no_function"))]
 impl AsRef<crate::Module> for AST {
     #[inline(always)]
-    #[must_use]
     fn as_ref(&self) -> &crate::Module {
         self.shared_lib().as_ref()
     }
@@ -837,7 +834,6 @@ impl AsRef<crate::Module> for AST {
 #[cfg(not(feature = "no_function"))]
 impl Borrow<crate::SharedModule> for AST {
     #[inline(always)]
-    #[must_use]
     fn borrow(&self) -> &crate::SharedModule {
         self.shared_lib()
     }
@@ -846,7 +842,6 @@ impl Borrow<crate::SharedModule> for AST {
 #[cfg(not(feature = "no_function"))]
 impl AsRef<crate::SharedModule> for AST {
     #[inline(always)]
-    #[must_use]
     fn as_ref(&self) -> &crate::SharedModule {
         self.shared_lib()
     }
@@ -917,14 +912,14 @@ impl ASTNode<'_> {
 /// _(internals)_ Encapsulated AST environment.
 /// Exported under the `internals` feature only.
 ///
-/// 1) functions defined within the same AST
+/// 1) stack of scripted functions defined
 /// 2) the stack of imported [modules][crate::Module]
 /// 3) global constants
 #[derive(Debug, Clone)]
 pub struct EncapsulatedEnviron {
-    /// Functions defined within the same [`AST`][crate::AST].
+    /// Stack of loaded [modules][crate::Module] containing script-defined functions.
     #[cfg(not(feature = "no_function"))]
-    pub lib: crate::SharedModule,
+    pub lib: StaticVec<crate::SharedModule>,
     /// Imported [modules][crate::Module].
     #[cfg(not(feature = "no_module"))]
     pub imports: crate::ThinVec<(ImmutableString, crate::SharedModule)>,
@@ -933,3 +928,19 @@ pub struct EncapsulatedEnviron {
     #[cfg(not(feature = "no_function"))]
     pub constants: Option<crate::eval::SharedGlobalConstants>,
 }
+
+#[cfg(not(feature = "no_function"))]
+impl From<&crate::eval::GlobalRuntimeState> for EncapsulatedEnviron {
+    fn from(value: &crate::eval::GlobalRuntimeState) -> Self {
+        Self {
+            lib: value.lib.clone(),
+            #[cfg(not(feature = "no_module"))]
+            imports: value
+                .iter_imports_raw()
+                .map(|(n, m)| (n.clone(), m.clone()))
+                .collect(),
+            #[cfg(not(feature = "no_module"))]
+            constants: value.constants.clone(),
+        }
+    }
+}
