@@ -2,6 +2,7 @@
 
 use super::{ASTFlags, ASTNode, Ident, Stmt, StmtBlock};
 use crate::engine::KEYWORD_FN_PTR;
+use crate::eval::GlobalRuntimeState;
 use crate::tokenizer::Token;
 use crate::types::dynamic::Union;
 use crate::{
@@ -223,7 +224,7 @@ impl fmt::Debug for FnCallExpr {
 impl FnCallExpr {
     /// Does this function call contain a qualified namespace?
     ///
-    /// Not available under [`no_module`]
+    /// Not available under `no_module`
     #[cfg(not(feature = "no_module"))]
     #[inline(always)]
     #[must_use]
@@ -329,11 +330,11 @@ pub enum Expr {
     /// * [`BREAK`][ASTFlags::BREAK] = terminate the chain (recurse into the chain if unset)
     Index(Box<BinaryExpr>, ASTFlags, Position),
     /// lhs `&&` rhs
-    And(Box<BinaryExpr>, Position),
+    And(Box<StaticVec<Expr>>, Position),
     /// lhs `||` rhs
-    Or(Box<BinaryExpr>, Position),
+    Or(Box<StaticVec<Expr>>, Position),
     /// lhs `??` rhs
-    Coalesce(Box<BinaryExpr>, Position),
+    Coalesce(Box<StaticVec<Expr>>, Position),
     /// Custom syntax
     #[cfg(not(feature = "no_custom_syntax"))]
     Custom(Box<CustomExpr>, Position),
@@ -341,7 +342,6 @@ pub enum Expr {
 
 impl Default for Expr {
     #[inline(always)]
-    #[must_use]
     fn default() -> Self {
         Self::Unit(Position::NONE)
     }
@@ -448,10 +448,11 @@ impl fmt::Debug for Expr {
                     display_pos = *pos;
                 }
 
-                f.debug_struct(op_name)
-                    .field("lhs", &x.lhs)
-                    .field("rhs", &x.rhs)
-                    .finish()
+                let mut f = f.debug_tuple(op_name);
+                x.iter().for_each(|expr| {
+                    f.field(expr);
+                });
+                f.finish()
             }
             #[cfg(not(feature = "no_custom_syntax"))]
             Self::Custom(x, ..) => f.debug_tuple("Custom").field(x).finish(),
@@ -467,9 +468,22 @@ impl Expr {
     /// Returns [`None`] if the expression is not a literal constant.
     #[inline]
     #[must_use]
-    pub fn get_literal_value(&self) -> Option<Dynamic> {
+    pub fn get_literal_value(&self, global: Option<&GlobalRuntimeState>) -> Option<Dynamic> {
         Some(match self {
-            Self::DynamicConstant(x, ..) => x.as_ref().clone(),
+            Self::DynamicConstant(x, ..) => {
+                let mut _value = x.as_ref().clone();
+
+                if let Some(global) = global {
+                    #[cfg(not(feature = "no_function"))]
+                    if let Some(mut fn_ptr) = _value.write_lock::<FnPtr>() {
+                        // Create a new environment with the current module
+                        fn_ptr.env = Some(crate::Shared::new(global.into()));
+                    }
+                }
+
+                _value
+            }
+
             Self::IntegerConstant(x, ..) => (*x).into(),
             #[cfg(not(feature = "no_float"))]
             Self::FloatConstant(x, ..) => (*x).into(),
@@ -481,7 +495,7 @@ impl Expr {
             #[cfg(not(feature = "no_index"))]
             Self::Array(x, ..) if self.is_constant() => {
                 let mut arr = crate::Array::with_capacity(x.len());
-                arr.extend(x.iter().map(|v| v.get_literal_value().unwrap()));
+                arr.extend(x.iter().map(|v| v.get_literal_value(global).unwrap()));
                 Dynamic::from_array(arr)
             }
 
@@ -490,7 +504,7 @@ impl Expr {
                 let mut map = x.1.clone();
 
                 for (k, v) in &x.0 {
-                    *map.get_mut(k.as_str()).unwrap() = v.get_literal_value().unwrap();
+                    *map.get_mut(k.as_str()).unwrap() = v.get_literal_value(global).unwrap();
                 }
 
                 Dynamic::from_map(map)
@@ -500,7 +514,7 @@ impl Expr {
             Self::InterpolatedString(x, ..) if self.is_constant() => {
                 let mut s = SmartString::new_const();
                 for segment in x {
-                    let v = segment.get_literal_value().unwrap();
+                    let v = segment.get_literal_value(global).unwrap();
                     write!(&mut s, "{v}").unwrap();
                 }
                 s.into()
@@ -694,11 +708,9 @@ impl Expr {
                 }
             }
 
-            Self::And(x, ..)
-            | Self::Or(x, ..)
-            | Self::Coalesce(x, ..)
-            | Self::Index(x, ..)
-            | Self::Dot(x, ..) => x.lhs.start_position(),
+            Self::And(x, ..) | Self::Or(x, ..) | Self::Coalesce(x, ..) => x[0].start_position(),
+
+            Self::Index(x, ..) | Self::Dot(x, ..) => x.lhs.start_position(),
 
             Self::FnCall(.., pos) => *pos,
 
@@ -752,7 +764,7 @@ impl Expr {
             Self::Map(x, ..) => x.0.iter().map(|(.., v)| v).all(Self::is_pure),
 
             Self::And(x, ..) | Self::Or(x, ..) | Self::Coalesce(x, ..) => {
-                x.lhs.is_pure() && x.rhs.is_pure()
+                x.iter().all(Expr::is_pure)
             }
 
             Self::Stmt(x) => x.iter().all(Stmt::is_pure),
@@ -879,11 +891,7 @@ impl Expr {
                     }
                 }
             }
-            Self::Index(x, ..)
-            | Self::Dot(x, ..)
-            | Self::And(x, ..)
-            | Self::Or(x, ..)
-            | Self::Coalesce(x, ..) => {
+            Self::Index(x, ..) | Self::Dot(x, ..) => {
                 if !x.lhs.walk(path, on_node) {
                     return false;
                 }
@@ -891,6 +899,13 @@ impl Expr {
                     return false;
                 }
             }
+            Self::And(x, ..) | Self::Or(x, ..) | Self::Coalesce(x, ..) => {
+                for expr in &***x {
+                    if !expr.walk(path, on_node) {
+                        return false;
+                    }
+                }
+            }
             Self::FnCall(x, ..) => {
                 for e in &*x.args {
                     if !e.walk(path, on_node) {
