@@ -511,8 +511,8 @@ impl Engine {
             }
             // Short-circuit for indexing with literal: {expr}[1]
             #[cfg(not(feature = "no_index"))]
-            (_, ChainType::Indexing) if rhs.get_literal_value().is_some() => {
-                idx_values.push(rhs.get_literal_value().unwrap())
+            (_, ChainType::Indexing) if rhs.get_literal_value(Some(global)).is_some() => {
+                idx_values.push(rhs.get_literal_value(Some(global)).unwrap())
             }
             // Short-circuit for simple method call: {expr}.func()
             #[cfg(not(feature = "no_object"))]
