@@ -68,17 +68,30 @@ impl Engine {
 
                 // Scripted function with the same name
                 #[cfg(not(feature = "no_function"))]
-                if let Some(fn_def) = global
+                if let Some(func) = global
                     .lib
                     .iter()
-                    .flat_map(|m| m.iter_script_fn())
-                    .find_map(|(_, _, f, _, func)| if f == v.1 { Some(func) } else { None })
+                    .flat_map(|m| m.iter_fn())
+                    .filter(|(f, _)| f.is_script())
+                    .filter(|(_, m)| m.name == v.1.as_str())
+                    .map(|(f, _)| f)
+                    .next()
                 {
+                    // Embedded environment for scripted function
+                    let env = if let Some(env) = func.get_shared_encapsulated_environ() {
+                        env.clone()
+                    } else {
+                        // Create a new environment with the current module
+                        crate::Shared::new((&*global).into())
+                    };
+
                     let val: Dynamic = crate::FnPtr {
                         name: v.1.clone(),
                         curry: <_>::default(),
-                        env: None,
-                        typ: crate::types::fn_ptr::FnPtrType::Script(fn_def.clone()),
+                        env: Some(env),
+                        typ: crate::types::fn_ptr::FnPtrType::Script(
+                            func.get_script_fn_def().cloned().unwrap(),
+                        ),
                     }
                     .into();
                     return Ok(val.into());
@@ -231,17 +244,26 @@ impl Engine {
         #[cfg(feature = "debugging")]
         defer! { global if Some(reset) => move |g| g.debugger_mut().reset_status(reset) }
 
-        match expr {
-            // Constants
-            Expr::IntegerConstant(x, ..) => Ok((*x).into()),
-            Expr::StringConstant(x, ..) => Ok(x.clone().into()),
-            Expr::BoolConstant(x, ..) => Ok((*x).into()),
+        // Constants
+        let is_simple_constant = match expr {
+            Expr::DynamicConstant(..)
+            | Expr::IntegerConstant(..)
+            | Expr::CharConstant(..)
+            | Expr::StringConstant(..)
+            | Expr::BoolConstant(..)
+            | Expr::Unit(..) => true,
             #[cfg(not(feature = "no_float"))]
-            Expr::FloatConstant(x, ..) => Ok((*x).into()),
-            Expr::CharConstant(x, ..) => Ok((*x).into()),
-            Expr::Unit(..) => Ok(Dynamic::UNIT),
-            Expr::DynamicConstant(x, ..) => Ok(x.as_ref().clone()),
+            Expr::FloatConstant(..) => true,
+            _ => false,
+        };
+        if is_simple_constant {
+            if let Some(value) = expr.get_literal_value(Some(global)) {
+                return Ok(value);
+            }
+        }
 
+        // Other expressions
+        match expr {
             Expr::FnCall(x, pos) => {
                 self.eval_fn_call_expr(global, caches, scope, this_ptr, x, *pos)
             }
@@ -341,35 +363,51 @@ impl Engine {
                 Ok(Dynamic::from_map(map))
             }
 
-            Expr::And(x, ..) => Ok((self
-                .eval_expr(global, caches, scope, this_ptr.as_deref_mut(), &x.lhs)?
-                .as_bool()
-                .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, x.lhs.position()))?
-                && self
-                    .eval_expr(global, caches, scope, this_ptr, &x.rhs)?
-                    .as_bool()
-                    .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, x.rhs.position()))?)
-            .into()),
+            Expr::And(x, ..) => {
+                let mut value = Dynamic::TRUE;
 
-            Expr::Or(x, ..) => Ok((self
-                .eval_expr(global, caches, scope, this_ptr.as_deref_mut(), &x.lhs)?
-                .as_bool()
-                .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, x.lhs.position()))?
-                || self
-                    .eval_expr(global, caches, scope, this_ptr, &x.rhs)?
-                    .as_bool()
-                    .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, x.rhs.position()))?)
-            .into()),
+                for expr in &***x {
+                    if !self
+                        .eval_expr(global, caches, scope, this_ptr.as_deref_mut(), expr)?
+                        .as_bool()
+                        .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, expr.position()))?
+                    {
+                        value = Dynamic::FALSE;
+                        break;
+                    }
+                }
+
+                Ok(value)
+            }
+
+            Expr::Or(x, ..) => {
+                let mut value = Dynamic::FALSE;
+
+                for expr in &***x {
+                    if self
+                        .eval_expr(global, caches, scope, this_ptr.as_deref_mut(), expr)?
+                        .as_bool()
+                        .map_err(|typ| self.make_type_mismatch_err::<bool>(typ, expr.position()))?
+                    {
+                        value = Dynamic::TRUE;
+                        break;
+                    }
+                }
+
+                Ok(value)
+            }
 
             Expr::Coalesce(x, ..) => {
-                let value =
-                    self.eval_expr(global, caches, scope, this_ptr.as_deref_mut(), &x.lhs)?;
+                let mut value = Dynamic::UNIT;
 
-                if value.is_unit() {
-                    self.eval_expr(global, caches, scope, this_ptr, &x.rhs)
-                } else {
-                    Ok(value)
+                for expr in &***x {
+                    value = self.eval_expr(global, caches, scope, this_ptr.as_deref_mut(), expr)?;
+
+                    if !value.is_unit() {
+                        break;
+                    }
                 }
+                Ok(value)
             }
 
             #[cfg(not(feature = "no_custom_syntax"))]
