@@ -1,6 +1,6 @@
 //! Type to hold a mutable reference to the target of an evaluation.
 
-use crate::{Dynamic, EvalAltResult, Position, RhaiError, RhaiResultOf};
+use crate::{Dynamic, Position, RhaiError, RhaiResultOf};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
 use std::{
@@ -404,6 +404,8 @@ impl<'a> Target<'a> {
                 let ve = if *exclusive {
                     let end = if *end > n { n } else { *end };
                     s.chars().skip(end)
+                } else if n == 0 {
+                    s.chars().skip(usize::MAX)
                 } else {
                     let end = if *end >= n { n - 1 } else { *end };
                     s.chars().skip(end + 1)
@@ -426,7 +428,9 @@ impl<'a> TryFrom<&'a mut Dynamic> for Target<'a> {
             // Cloning is cheap for a shared value
             let shared_value = value.clone();
             let Some(guard) = value.write_lock::<Dynamic>() else {
-                return Err(EvalAltResult::ErrorDataRace(String::new(), Position::NONE).into());
+                return Err(
+                    crate::EvalAltResult::ErrorDataRace(String::new(), Position::NONE).into(),
+                );
             };
             return Ok(Self::SharedValue {
                 guard,
