@@ -655,7 +655,6 @@ pub struct Module {
 
 impl Default for Module {
     #[inline(always)]
-    #[must_use]
     fn default() -> Self {
         Self::new()
     }
@@ -2323,17 +2322,18 @@ impl Module {
         let mut module = Self::new();
 
         // Extra modules left become sub-modules
-        let mut imports = crate::ThinVec::new();
-
-        if result.is_ok() {
+        let imports = if result.is_ok() {
             global
                 .scan_imports_raw()
                 .skip(orig_imports_len)
-                .for_each(|(k, m)| {
-                    imports.push((k.clone(), m.clone()));
-                    module.set_sub_module(k.clone(), m.clone());
-                });
-        }
+                .map(|(k, m)| (k.clone(), m.clone()))
+                .collect()
+        } else {
+            crate::ThinVec::new()
+        };
+        imports.iter().for_each(|(k, m)| {
+            module.set_sub_module(k.clone(), m.clone());
+        });
 
         // Restore global state
         #[cfg(not(feature = "no_function"))]
@@ -2353,7 +2353,7 @@ impl Module {
         #[cfg(not(feature = "no_function"))]
         let env = Shared::new(crate::ast::EncapsulatedEnviron {
             #[cfg(not(feature = "no_function"))]
-            lib: ast.shared_lib().clone(),
+            lib: std::iter::once(ast.shared_lib().clone()).collect(),
             imports,
             #[cfg(not(feature = "no_function"))]
             constants,
