@@ -4,6 +4,7 @@ use crate::api::deprecated::deprecated_array_functions;
 use crate::engine::OP_EQUALS;
 use crate::eval::{calc_index, calc_offset_len};
 use crate::plugin::*;
+use crate::types::fn_ptr::FnPtrType;
 use crate::{
     def_package, Array, Dynamic, ExclusiveRange, FnPtr, InclusiveRange, NativeCallContext,
     Position, RhaiResultOf, ERR, INT, MAX_USIZE_INT,
@@ -1271,9 +1272,9 @@ pub mod array_functions {
         let comparer = FnPtr {
             name: ctx.engine().get_interned_string(OP_EQUALS),
             curry: <_>::default(),
-            environ: None,
             #[cfg(not(feature = "no_function"))]
-            fn_def: None,
+            env: None,
+            typ: FnPtrType::Normal,
         };
         dedup_by_comparer(ctx, array, comparer);
     }
@@ -1638,8 +1639,8 @@ pub mod array_functions {
         }
         if type_id == TypeId::of::<ImmutableString>() {
             array.sort_by(|a, b| {
-                let a = &*a.read_lock::<ImmutableString>().unwrap();
-                let b = &*b.read_lock::<ImmutableString>().unwrap();
+                let a = &*a.as_immutable_string_ref().unwrap();
+                let b = &*b.as_immutable_string_ref().unwrap();
                 a.cmp(b)
             });
             return Ok(());
