@@ -73,7 +73,12 @@ impl fmt::Debug for FnPtr {
     #[cold]
     #[inline(never)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let ff = &mut f.debug_tuple(&self.typ.to_string());
+        let name = match self {
+            #[cfg(not(feature = "no_function"))]
+            _ if self.env.is_some() => format!("{}+", self.typ),
+            _ => self.typ.to_string(),
+        };
+        let ff = &mut f.debug_tuple(&name);
         ff.field(&self.name);
         self.curry.iter().for_each(|curry| {
             ff.field(curry);
@@ -352,7 +357,7 @@ impl FnPtr {
                     fn_def,
                     args,
                     true,
-                    context.position(),
+                    context.call_position(),
                 );
             }
             _ => (),
@@ -501,7 +506,7 @@ impl FnPtr {
             .map_err(|err| {
                 Box::new(ERR::ErrorInFunctionCall(
                     fn_name.to_string(),
-                    ctx.source().unwrap_or("").to_string(),
+                    ctx.fn_source().unwrap_or("").to_string(),
                     err,
                     Position::NONE,
                 ))
